"use strict";
/*
* This program and the accompanying materials are made available under the terms of the *
* Eclipse Public License v2.0 which accompanies this distribution, and is available at *
* https://www.eclipse.org/legal/epl-v20.html                                      *
*                                                                                 *
* SPDX-License-Identifier: EPL-2.0                                                *
*                                                                                 *
* Copyright Contributors to the Zowe Project.                                     *
*                                                                                 *
*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const imperative_1 = require("@brightside/imperative");
const __1 = require("../");
const ibmdb = require("ibm_db");
const Messages_1 = require("./doc/Messages");
/**
 * Class to handle exporting of DB2 tables
 * @export
 * @class ExportTable
 */
class ExportTable {
    /**
     * Constructor
     * @param {IDB2Session} session DB2 session parameters
     * @param {string} databaseName Name of the database
     * @param {string} tableName Name of the table
     */
    constructor(session, databaseName, tableName) {
        __1.SessionValidator.validate(session);
        imperative_1.ImperativeExpect.toBeDefinedAndNonBlank(databaseName, Messages_1.noDatabaseName.message);
        imperative_1.ImperativeExpect.toBeDefinedAndNonBlank(tableName, Messages_1.noTableName.message);
        this.mDatabase = databaseName;
        this.mTable = tableName;
        this.mConnectionString = __1.ConnectionString.buildFromSession(session);
        this.mMetadata = null;
    }
    init() {
        return __awaiter(this, void 0, void 0, function* () {
            const options = {
                fetchMode: __1.DB2Constants.FETCH_MODE_OBJECT,
            };
            try {
                this.mConnection = ibmdb.openSync(this.mConnectionString, options);
            }
            catch (err) {
                throw new imperative_1.ImperativeError({ msg: err.toString() });
            }
            this.mMetadata = yield this.getTableMeta();
            if (Array.isArray(this.mMetadata) && this.mMetadata.length === 0) {
                throw new imperative_1.ImperativeError({ msg: `Error getting metadata for the table ${this.mDatabase}.${this.mTable}` });
            }
        });
    }
    /**
     * Get a table metadata
     * @returns {Promise<IDB2Column[]>}
     * @memberof ExportTable
     */
    getTableMeta() {
        return new Promise((resolve, reject) => {
            if (this.mMetadata == null) {
                this.mConnection.columns(null, this.mDatabase, this.mTable, null, (err, res) => {
                    if (err !== null) {
                        reject(err);
                    }
                    resolve(res);
                });
            }
            else {
                resolve(this.mMetadata);
            }
        });
    }
    getColumnMeta(columnName) {
        for (const meta of this.mMetadata) {
            if (meta.COLUMN_NAME === columnName) {
                return meta;
            }
        }
        return null;
    }
    /**
     * Supply data from table by one row
     * @returns {IterableIterator<any>}
     */
    *rows() {
        const columns = this.getColumnNames().join(", ");
        const query = `SELECT ${columns} FROM ${this.mDatabase}.${this.mTable}`;
        const result = this.mConnection.queryResultSync(query);
        let row = result.fetchSync();
        while (row != null) {
            yield row;
            row = result.fetchSync();
        }
        result.closeSync();
    }
    /**
     * Extract a list of columns out of a table metadata
     * @returns {string[]} Columns list
     */
    getColumnNames() {
        const columnNames = [];
        for (const column of this.mMetadata) {
            columnNames.push(column.COLUMN_NAME);
        }
        return columnNames;
    }
}
exports.ExportTable = ExportTable;
//# sourceMappingURL=ExportTable.js.map