"use strict";
/*
* This program and the accompanying materials are made available under the terms of the *
* Eclipse Public License v2.0 which accompanies this distribution, and is available at *
* https://www.eclipse.org/legal/epl-v20.html                                      *
*                                                                                 *
* SPDX-License-Identifier: EPL-2.0                                                *
*                                                                                 *
* Copyright Contributors to the Zowe Project.                                     *
*                                                                                 *
*/
Object.defineProperty(exports, "__esModule", { value: true });
const imperative_1 = require("@brightside/imperative");
const __1 = require("../");
const util_1 = require("util");
/**
 * Class to handle exporting DB2 tables
 * @export
 * @class ExportTable
 */
class ExportTableSQL extends __1.ExportTable {
    /**
     * Constructor
     * @param {IDB2Session} session DB2 session parameters
     * @param {string} databaseName Name of the database
     * @param {string} tableName Name of the table
     */
    constructor(session, databaseName, tableName) {
        super(session, databaseName, tableName);
    }
    /**
     * Export a DB2 table in SQL INSERT format
     * @memberof ExportTableSQL
     */
    *export() {
        try {
            let row;
            const rows = this.rows();
            while (!(row = rows.next()).done) {
                yield this.buildInsertStatement(row.value);
            }
        }
        catch (err) {
            throw new imperative_1.ImperativeError({ msg: err.toString() });
        }
    }
    /**
     * Make an SQL INSERT statement out of the table metadata and values
     * @param {any} data Object containing row data
     * @returns {string}
     */
    buildInsertStatement(data) {
        const columnNames = this.getColumnNames();
        const values = [];
        for (const columnName of columnNames) {
            values.push(this.escape(columnName, data[columnName]));
        }
        const columnList = columnNames.join(", ");
        const valuesList = values.join(", ");
        return `INSERT INTO ${this.mDatabase}.${this.mTable}(${columnList})\nVALUES(${valuesList})\n`;
    }
    /**
     * Add quotes to character types
     * @param {string} columnName Name of the column
     * @param value Value to escape
     * @returns {any}
     */
    escape(columnName, value) {
        const columnMeta = this.getColumnMeta(columnName);
        if (util_1.isNull(value)) {
            return "NULL";
        }
        switch (columnMeta.TYPE_NAME) {
            case "CHAR":
            case "DATE":
            case "TIME":
            case "TIMESTAMP":
            case "VARCHAR":
                return `'${value}'`;
            default:
                return value;
        }
    }
}
exports.ExportTableSQL = ExportTableSQL;
//# sourceMappingURL=ExportTableSQL.js.map