"use strict";
/*
* This program and the accompanying materials are made available under the terms of the
* Eclipse Public License v2.0 which accompanies this distribution, and is available at
* https://www.eclipse.org/legal/epl-v20.html
*
* SPDX-License-Identifier: EPL-2.0
*
* Copyright Contributors to the Zowe Project.
*
*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const imperative_1 = require("@brightside/imperative");
const fs = require("fs");
const path = require("path");
const rest_1 = require("../../../../../rest");
const ZosFiles_constants_1 = require("../../constants/ZosFiles.constants");
const ZosFiles_messages_1 = require("../../constants/ZosFiles.messages");
const ZosFilesUtils_1 = require("../../utils/ZosFilesUtils");
const list_1 = require("../list");
const create_1 = require("../create");
class Upload {
    /**
     * Upload content from file to data set
     * @param {AbstractSession} session      - z/OS connection info
     * @param {string}          inputFile    - path to a file
     * @param {string}          dataSetName  - Name of the data set to write to
     * @param {IUploadOptions}  [options={}] - Uploading options
     *
     * @return {Promise<IZosFilesResponse>} A response indicating the out come
     *
     * @throws {ImperativeError} When encounter error scenarios.
     *
     */
    static fileToDataset(session, inputFile, dataSetName, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            this.log.info(`Uploading file ${inputFile} to ${dataSetName}`);
            imperative_1.ImperativeExpect.toNotBeNullOrUndefined(inputFile, ZosFiles_messages_1.ZosFilesMessages.missingInputFile.message);
            imperative_1.ImperativeExpect.toNotBeNullOrUndefined(dataSetName, ZosFiles_messages_1.ZosFilesMessages.missingDatasetName.message);
            imperative_1.ImperativeExpect.toNotBeEqual(dataSetName, "", ZosFiles_messages_1.ZosFilesMessages.missingDatasetName.message);
            const promise = new Promise((resolve, reject) => {
                fs.lstat(inputFile, (err, stats) => {
                    if (err == null && stats.isFile()) {
                        resolve(true);
                    }
                    else if (err == null) {
                        reject(new imperative_1.ImperativeError({
                            msg: ZosFiles_messages_1.ZosFilesMessages.missingInputFile.message
                        }));
                    }
                    else {
                        reject(new imperative_1.ImperativeError({
                            msg: ZosFiles_messages_1.ZosFilesMessages.nodeJsFsError.message,
                            additionalDetails: err.toString(),
                            causeErrors: err
                        }));
                    }
                });
            });
            yield promise;
            return this.pathToDataSet(session, inputFile, dataSetName, options);
        });
    }
    /**
     * Upload content from a directory to a PDS
     * @param {AbstractSession} session      - z/OS connection info
     * @param {string}          inputDir     - path to a file
     * @param {string}          dataSetName  - Name of the data set to write to
     * @param {IUploadOptions}  [options={}] - Uploading options
     *
     * @return {Promise<IZosFilesResponse>} A response indicating the out come
     *
     * @throws {ImperativeError} When encounter error scenarios.
     *
     */
    static dirToPds(session, inputDir, dataSetName, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            this.log.info(`Uploading directory ${inputDir} to ${dataSetName}`);
            imperative_1.ImperativeExpect.toNotBeNullOrUndefined(inputDir, ZosFiles_messages_1.ZosFilesMessages.missingInputDir.message);
            imperative_1.ImperativeExpect.toNotBeNullOrUndefined(dataSetName, ZosFiles_messages_1.ZosFilesMessages.missingDatasetName.message);
            imperative_1.ImperativeExpect.toNotBeEqual(dataSetName, "", ZosFiles_messages_1.ZosFilesMessages.missingDatasetName.message);
            const promise = new Promise((resolve, reject) => {
                fs.lstat(inputDir, (err, stats) => {
                    if (err == null && !stats.isFile()) {
                        resolve(true);
                    }
                    else if (err == null) {
                        reject(new imperative_1.ImperativeError({
                            msg: ZosFiles_messages_1.ZosFilesMessages.pathIsNotDirectory.message
                        }));
                    }
                    else {
                        reject(new imperative_1.ImperativeError({
                            msg: ZosFiles_messages_1.ZosFilesMessages.nodeJsFsError.message,
                            additionalDetails: err.toString(),
                            causeErrors: err
                        }));
                    }
                });
            });
            yield promise;
            if (!imperative_1.IO.isDir(inputDir)) {
                throw new imperative_1.ImperativeError({
                    msg: ZosFiles_messages_1.ZosFilesMessages.missingInputDir.message
                });
            }
            return this.pathToDataSet(session, inputDir, dataSetName, options);
        });
    }
    /**
     * Writting data buffer to a data set.
     * @param {AbstractSession} session      - z/OS connection info
     * @param {Buffer}          fileBuffer   - Data buffer to be written
     * @param {string}          dataSetName  - Name of the data set to write to
     * @param {IUploadOptions}  [options={}] - Uploading options
     *
     * @return {Promise<IZosFilesResponse>} A response indicating the out come
     *
     * @throws {ImperativeError} When encounter error scenarios.
     */
    static bufferToDataSet(session, fileBuffer, dataSetName, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            imperative_1.ImperativeExpect.toNotBeNullOrUndefined(dataSetName, ZosFiles_messages_1.ZosFilesMessages.missingDatasetName.message);
            try {
                // Construct zOSMF REST endpoint.
                let endpoint = path.posix.join(ZosFiles_constants_1.ZosFilesConstants.RESOURCE, ZosFiles_constants_1.ZosFilesConstants.RES_DS_FILES);
                if (options.volume) {
                    endpoint = path.posix.join(endpoint, `-(${options.volume})`);
                }
                endpoint = path.posix.join(endpoint, dataSetName);
                // Construct request header parameters
                const reqHeader = [];
                if (options.binary) {
                    reqHeader.push(rest_1.ZosmfHeaders.X_IBM_BINARY);
                }
                else {
                    reqHeader.push(rest_1.ZosmfHeaders.X_IBM_TEXT);
                    fileBuffer = ZosFilesUtils_1.ZosFilesUtils.normalizeNewline(fileBuffer);
                }
                // Migrated recall options
                if (options.recall) {
                    switch (options.recall.toLowerCase()) {
                        case "wait":
                            reqHeader.push(rest_1.ZosmfHeaders.X_IBM_MIGRATED_RECALL_WAIT);
                            break;
                        case "nowait":
                            reqHeader.push(rest_1.ZosmfHeaders.X_IBM_MIGRATED_RECALL_NO_WAIT);
                            break;
                        case "error":
                            reqHeader.push(rest_1.ZosmfHeaders.X_IBM_MIGRATED_RECALL_ERROR);
                            break;
                        default:
                            reqHeader.push(rest_1.ZosmfHeaders.X_IBM_MIGRATED_RECALL_NO_WAIT);
                            break;
                    }
                }
                yield rest_1.ZosmfRestClient.putExpectString(session, endpoint, reqHeader, fileBuffer);
                return {
                    success: true,
                    commandResponse: ZosFiles_messages_1.ZosFilesMessages.dataSetUploadedSuccessfully.message
                };
            }
            catch (error) {
                throw error;
            }
        });
    }
    /**
     * Upload content from input path to dataSet or PDS members
     * @param {AbstractSession} session      - z/OS connection info
     * @param {string}          inputPath    - User input path to file or directory
     * @param {string}          dataSetName  - Name of the data set to write to
     * @param {IUploadOptions}  [options={}] - Uploading options
     *
     * @return {Promise<IZosFilesResponse>} A response indicating the out come
     *
     * @throws {ImperativeError} When encounter error scenarios.
     *
     * @example pathToDataSet(session, "file.txt", "ps.name")
     * @example pathToDataset(session, "file.txt", "psd.name(member)")
     * @example pathToDataSet(session, "directory", "pds.name")
     * @example pathToDataset(session, "/full/path/file.txt", "ps.name")
     *
     * Note:
     * This method does everything needed to do from checking if path is file or directory
     * and if data set is sequential file or PDS to determind what name to be used when
     * upload content to data set.  All you have to specify is a directory and a dsname.
     */
    static pathToDataSet(session, inputPath, dataSetName, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            this.log.info(`Uploading path ${inputPath} to ${dataSetName}`);
            imperative_1.ImperativeExpect.toNotBeNullOrUndefined(dataSetName, ZosFiles_messages_1.ZosFilesMessages.missingDatasetName.message);
            imperative_1.ImperativeExpect.toNotBeEqual(dataSetName, "", ZosFiles_messages_1.ZosFilesMessages.missingDatasetName.message);
            let payload;
            let memberName = "";
            let uploadingFile = "";
            let uploadingDsn = "";
            let uploadFileList;
            let isUploadToPds = false;
            const results = [];
            // Check and make sure that data set name does not contain any masking character (%*).
            if (ZosFilesUtils_1.ZosFilesUtils.isDataSetNameContainMasking(dataSetName)) {
                throw new imperative_1.ImperativeError({
                    msg: ZosFiles_messages_1.ZosFilesMessages.unsupportedMaskingInDataSetName.message
                });
            }
            // From the input path, retrieve the list of all files that we are trying to upload.
            // If input is a file, the return is an array of 1 element,
            // If input is a directory, the return will be an array of all of it files.
            uploadFileList = ZosFilesUtils_1.ZosFilesUtils.getFileListFromPath(inputPath);
            // Check if data set name is actually a PDS member.
            //   if it is, then we have to make sure that we do not try to upload a directory
            //   to a PDS member.
            if (dataSetName.endsWith(")")) {
                if (imperative_1.IO.isDir(inputPath)) {
                    throw new imperative_1.ImperativeError({
                        msg: ZosFiles_messages_1.ZosFilesMessages.uploadDirectoryToDatasetMember.message
                    });
                }
                memberName = dataSetName.substr(dataSetName.indexOf("(")).replace(/[()]/g, "");
                dataSetName = dataSetName.substr(0, dataSetName.indexOf("("));
            }
            // Retrieve the information on the input data set name to determine if it is a
            // sequential data set or PDS.
            const listResponse = yield list_1.List.dataSet(session, dataSetName, { attributes: true });
            if (listResponse.apiResponse != null && listResponse.apiResponse.returnedRows != null && listResponse.apiResponse.items != null) {
                // Look for the index of the data set in the response from the List API
                const dsnameIndex = listResponse.apiResponse.returnedRows === 0 ? -1 :
                    listResponse.apiResponse.items.findIndex((ds) => ds.dsname.toUpperCase() === dataSetName.toUpperCase());
                if (dsnameIndex !== -1) {
                    // If dsnameIndex === -1, it means we could not find the given data set.
                    // We will attempt the upload anyways so that we can forward/throw the proper error from z/OS MF
                    const dsInfo = listResponse.apiResponse.items[dsnameIndex];
                    switch (dsInfo.dsorg) {
                        case "PO":
                        case "PO-E":
                            isUploadToPds = true;
                            break;
                        default:
                            // if loading to a physical sequential data set and multiple files found then error
                            if (uploadFileList.length > 1) {
                                throw new imperative_1.ImperativeError({
                                    msg: ZosFiles_messages_1.ZosFilesMessages.uploadDirectoryToPhysicalSequentialDataSet.message
                                });
                            }
                            break;
                    }
                }
            }
            // Loop through the array of upload file and perform upload one file at a time.
            // The reason we can not perform this task in parallel is because the Enqueueing on a PDS.  It will
            // result will random errors when trying to upload to multiple member of the same PDS at the same time.
            // This also allow us to break out as soon as the first error is encounter instead of wait until the
            // entire list is processed.
            try {
                let uploadError;
                let uploadsInitiated = 0;
                for (const file of uploadFileList) {
                    uploadingFile = file;
                    uploadingDsn = dataSetName;
                    if (isUploadToPds) {
                        if (memberName.length === 0) {
                            uploadingDsn = `${uploadingDsn}(${ZosFilesUtils_1.ZosFilesUtils.generateMemberName(uploadingFile)})`;
                        }
                        else {
                            uploadingDsn = `${uploadingDsn}(${memberName})`;
                        }
                    }
                    uploadingDsn = uploadingDsn.toUpperCase();
                    if (options.task != null) {
                        // update the progress bar if any
                        const LAST_FIFTEEN_CHARS = -15;
                        const abbreviatedFile = uploadingFile.slice(LAST_FIFTEEN_CHARS);
                        options.task.statusMessage = "Uploading ..." + abbreviatedFile;
                        options.task.percentComplete = Math.floor(imperative_1.TaskProgress.ONE_HUNDRED_PERCENT *
                            (uploadsInitiated / uploadFileList.length));
                        uploadsInitiated++;
                    }
                    if (uploadError === undefined) {
                        try {
                            // read payload from file
                            payload = fs.readFileSync(uploadingFile);
                            const result = yield this.bufferToDataSet(session, payload, uploadingDsn, options);
                            this.log.info(`Success Uploaded data From ${uploadingFile} To ${uploadingDsn}`);
                            results.push({
                                success: result.success,
                                from: uploadingFile,
                                to: uploadingDsn
                            });
                        }
                        catch (err) {
                            this.log.error(`Failure Uploading data From ${uploadingFile} To ${uploadingDsn}`);
                            results.push({
                                success: false,
                                from: uploadingFile,
                                to: uploadingDsn,
                                error: err
                            });
                            uploadError = err;
                        }
                    }
                    else {
                        results.push({
                            success: undefined,
                            from: uploadingFile,
                            to: uploadingDsn,
                        });
                    }
                }
                if (uploadError) {
                    throw uploadError;
                }
                return {
                    success: true,
                    commandResponse: ZosFiles_messages_1.ZosFilesMessages.dataSetUploadedSuccessfully.message,
                    apiResponse: results
                };
            }
            catch (error) {
                return {
                    success: false,
                    commandResponse: error.message,
                    apiResponse: results
                };
            }
        });
    }
    /**
     * Upload content to USS file
     * @param {AbstractSession} session - z/OS connection info
     * @param {string} ussname          - Name of the USS file to write to
     * @param {Buffer} buffer          - Data to be written
     * @param {boolean} binary          - The indicator to upload the file in binary mode
     * @returns {Promise<object>}
     */
    static bufferToUSSFile(session, ussname, buffer, binary = false) {
        return __awaiter(this, void 0, void 0, function* () {
            imperative_1.ImperativeExpect.toNotBeNullOrUndefined(ussname, ZosFiles_messages_1.ZosFilesMessages.missingUSSFileName.message);
            ussname = path.posix.normalize(ussname);
            ussname = Upload.formatUnixFilepath(ussname);
            const parameters = ZosFiles_constants_1.ZosFilesConstants.RES_USS_FILES + "/" + ussname;
            const headers = [];
            if (binary) {
                headers.push(rest_1.ZosmfHeaders.OCTET_STREAM);
                headers.push(rest_1.ZosmfHeaders.X_IBM_BINARY);
            }
            else {
                headers.push(rest_1.ZosmfHeaders.TEXT_PLAIN);
            }
            return rest_1.ZosmfRestClient.putExpectString(session, ZosFiles_constants_1.ZosFilesConstants.RESOURCE + parameters, headers, buffer);
        });
    }
    static fileToUSSFile(session, inputFile, ussname, binary = false) {
        return __awaiter(this, void 0, void 0, function* () {
            imperative_1.ImperativeExpect.toNotBeNullOrUndefined(inputFile, ZosFiles_messages_1.ZosFilesMessages.missingInputFile.message);
            imperative_1.ImperativeExpect.toNotBeNullOrUndefined(ussname, ZosFiles_messages_1.ZosFilesMessages.missingUSSFileName.message);
            imperative_1.ImperativeExpect.toNotBeEqual(ussname, "", ZosFiles_messages_1.ZosFilesMessages.missingUSSFileName.message);
            const promise = new Promise((resolve, reject) => {
                fs.lstat(inputFile, (err, stats) => {
                    if (err == null && stats.isFile()) {
                        resolve(true);
                    }
                    else if (err == null) {
                        reject(new imperative_1.ImperativeError({
                            msg: ZosFiles_messages_1.ZosFilesMessages.missingInputFile.message
                        }));
                    }
                    else {
                        reject(new imperative_1.ImperativeError({
                            msg: ZosFiles_messages_1.ZosFilesMessages.nodeJsFsError.message,
                            additionalDetails: err.toString(),
                            causeErrors: err
                        }));
                    }
                });
            });
            yield promise;
            let payload;
            let result;
            // read payload from file
            payload = fs.readFileSync(inputFile);
            yield this.bufferToUSSFile(session, ussname, payload, binary);
            result = {
                success: true,
                from: inputFile,
                to: ussname
            };
            return {
                success: true,
                commandResponse: ZosFiles_messages_1.ZosFilesMessages.ussFileUploadedSuccessfully.message,
                apiResponse: result
            };
        });
    }
    /**
     * Upload local directory to USS directory
     * @param {AbstractSession} session - z/OS connection info
     * @param {string} inputDirectory   - the path of local directory
     * @param {string} ussname          - the name of uss folder
     * @param {boolean} binary          - the indicator to upload the file in binary mode
     * @param {boolean} recursive       - the indicator to upload local folder recursively
     * @param {IUploadMap} filesMap     - the map to define which files to upload in binary or asci mode
     * @returns {Promise<IZosFilesResponse>}
     */
    static dirToUSSDir(session, inputDirectory, ussname, binary = false, recursive = false, filesMap) {
        return __awaiter(this, void 0, void 0, function* () {
            imperative_1.ImperativeExpect.toNotBeNullOrUndefined(inputDirectory, ZosFiles_messages_1.ZosFilesMessages.missingInputDirectory.message);
            imperative_1.ImperativeExpect.toNotBeEqual(inputDirectory, "", ZosFiles_messages_1.ZosFilesMessages.missingInputDirectory.message);
            imperative_1.ImperativeExpect.toNotBeNullOrUndefined(ussname, ZosFiles_messages_1.ZosFilesMessages.missingUSSDirectoryName.message);
            imperative_1.ImperativeExpect.toNotBeEqual(ussname, "", ZosFiles_messages_1.ZosFilesMessages.missingUSSDirectoryName.message);
            // Check if inputDirectory is directory
            if (!imperative_1.IO.isDir(inputDirectory)) {
                throw new imperative_1.ImperativeError({
                    msg: ZosFiles_messages_1.ZosFilesMessages.missingInputDirectory.message
                });
            }
            // Check if provided unix directory exists
            const isDirectoryExist = yield this.isDirectoryExist(session, ussname);
            if (!isDirectoryExist) {
                yield create_1.Create.uss(session, ussname, "directory");
            }
            if (recursive === false) {
                const files = ZosFilesUtils_1.ZosFilesUtils.getFileListFromPath(inputDirectory, false);
                yield Promise.all(files.map((fileName) => __awaiter(this, void 0, void 0, function* () {
                    const filePath = path.normalize(path.join(inputDirectory, fileName));
                    if (!imperative_1.IO.isDir(filePath)) {
                        let tempBinary;
                        if (filesMap) {
                            if (filesMap.fileNames.indexOf(fileName) > -1) {
                                tempBinary = filesMap.binary;
                            }
                            else {
                                tempBinary = binary;
                            }
                        }
                        else {
                            tempBinary = binary;
                        }
                        const ussFilePath = path.posix.join(ussname, fileName);
                        yield this.fileToUSSFile(session, filePath, ussFilePath, tempBinary);
                    }
                })));
            }
            else {
                yield this.dirToUSSDirRecursive(session, inputDirectory, ussname, binary, filesMap);
            }
            const result = {
                success: true,
                from: inputDirectory,
                to: ussname
            };
            return {
                success: true,
                commandResponse: ZosFiles_messages_1.ZosFilesMessages.ussDirUploadedSuccessfully.message,
                apiResponse: result
            };
        });
    }
    /**
     * Check if USS directory exists
     * @param {AbstractSession} session - z/OS connection info
     * @param {string} ussname          - the name of uss folder
     * @return {Promise<boolean>}
     */
    static isDirectoryExist(session, ussname) {
        return __awaiter(this, void 0, void 0, function* () {
            ussname = path.posix.normalize(ussname);
            const parameters = `${ZosFiles_constants_1.ZosFilesConstants.RES_USS_FILES}?path=${ussname}`;
            try {
                const response = yield rest_1.ZosmfRestClient.getExpectJSON(session, ZosFiles_constants_1.ZosFilesConstants.RESOURCE + parameters);
                if (response.items) {
                    return true;
                }
            }
            catch (err) {
                if (err) {
                    return false;
                }
            }
            return false;
        });
    }
    /**
     * Upload directory to USS recursively
     * @param {AbstractSession} session - z/OS connection info
     * @param {string} inputDirectory   - the path of local directory
     * @param {string} ussname          - the name of uss folder
     * @param {boolean} binary          - the indicator to upload the file in binary mode
     * @param {IUploadMap} filesMap     - the map to define which files to upload in binary or asci mode
     * @return {null}
     */
    static dirToUSSDirRecursive(session, inputDirectory, ussname, binary, filesMap) {
        return __awaiter(this, void 0, void 0, function* () {
            yield Promise.all(fs.readdirSync(inputDirectory).map((fileName) => __awaiter(this, void 0, void 0, function* () {
                const filePath = path.normalize(path.join(inputDirectory, fileName));
                if (!imperative_1.IO.isDir(filePath)) {
                    let tempBinary;
                    if (filesMap) {
                        if (filesMap.fileNames.indexOf(fileName) > -1) {
                            tempBinary = filesMap.binary;
                        }
                        else {
                            tempBinary = binary;
                        }
                    }
                    else {
                        tempBinary = binary;
                    }
                    const ussFilePath = path.posix.join(ussname, fileName);
                    yield this.fileToUSSFile(session, filePath, ussFilePath, tempBinary);
                }
                else {
                    const tempUssPath = path.posix.join(ussname, fileName);
                    // Check if provided unix directory exists
                    const isDirectoryExist = yield this.isDirectoryExist(session, tempUssPath);
                    if (!isDirectoryExist) {
                        yield create_1.Create.uss(session, tempUssPath, "directory");
                    }
                    yield this.dirToUSSDirRecursive(session, filePath, tempUssPath, binary, filesMap);
                }
            })));
        });
    }
    /**
     * Get Log
     * @returns {Logger} applicationLogger.
     */
    static get log() {
        return imperative_1.Logger.getAppLogger();
        // return Logger.getConsoleLogger();
    }
    /**
     * Format USS filepaths in the way that the APIs expect (no leading /)
     * @param {string} usspath - the path to format
     */
    static formatUnixFilepath(usspath) {
        if (usspath.charAt(0) === "/") {
            // trim leading slash from unix files - API doesn't like it
            usspath = usspath.substring(1);
        }
        return usspath;
    }
}
exports.Upload = Upload;
//# sourceMappingURL=Upload.js.map